

package com.flexnet.external.webservice.idgenerator;

import java.util.List;
import java.util.Random;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.jws.WebService;

import com.flexnet.external.type.BulkEntitlement;
import com.flexnet.external.type.ChannelPartner;
import com.flexnet.external.type.ConsolidatedLicenseRecord;
import com.flexnet.external.type.CustomAttribute;
import com.flexnet.external.type.Entitlement;
import com.flexnet.external.type.EntitlementLineItem;
import com.flexnet.external.type.FulfillmentRecord;
import com.flexnet.external.type.Id;
import com.flexnet.external.type.LicenseModel;
import com.flexnet.external.type.MaintenanceItem;
import com.flexnet.external.type.OrganizationUnit;
import com.flexnet.external.type.PartNumber;
import com.flexnet.external.type.PingRequest;
import com.flexnet.external.type.PingResponse;
import com.flexnet.external.type.Product;
import com.flexnet.external.type.SvcException;

@WebService(
    endpointInterface="com.flexnet.external.webservice.idgenerator.IdGeneratorServiceInterface",
    wsdlLocation = "/schema/IdGeneratorService.wsdl"
)


public class IdGeneratorServiceImpl implements IdGeneratorServiceInterface{
    private Random randomGenerator = new Random();
    
  
    private String generateUniqueRandom(String prefix, Integer offset)  {
        String timeElapsed = Long.toHexString(System.currentTimeMillis());
        System.out.println("UniqueRandom " + timeElapsed);
        return prefix + getQuadruplets( timeElapsed.substring(0, 5) + String.format("%04X", offset) + timeElapsed.substring(5, timeElapsed.length()), '-');    
    }
    
    private String getQuadruplets(String input, char separator)  {
        if (input == null || input == "")
            return null;
        
        StringBuilder builder = new StringBuilder();
        String str =  new StringBuilder(input).reverse().toString();
        int i = 0;
        for(char c : str.toCharArray()) {
            if (i % 4 == 0 && i != 0)  
                builder.append(separator);
            builder.append(c);
            i++;
        }        
        return builder.toString();
    }
    
    private String getMagicChars(String str)  {
        StringBuilder builder = new StringBuilder();
        int i = 0;
        for(char c : str.toCharArray()) {
            if (i == 0 || Character.isUpperCase(c))    {
                builder.append(c);
                i = 1;
                continue;
            }
            if( c == '_' || c == ' ' && c == '-') 
                i = -1;
            else 
                ++i;
        }
        return builder.toString();
    }
    
    private String getEntitlementId(Entitlement entitlement)  {
        List<ChannelPartner> partners = entitlement.getEntChannelPartners();
        OrganizationUnit ou = entitlement.getSoldTo();
        List<CustomAttribute> attr = entitlement.getCustomAttributes();
        if (ou != null )  {
            return getMagicChars(ou.getName()) + '-';
        }
        else if (partners != null && partners.size() >= 1)  {
            return getMagicChars(partners.get(0).getOrgUnit().getDisplayName()) + '-';
        }
        else if (attr != null && attr.size() >= 1)  {
            return getMagicChars( attr.get(0).getName()) + '-';
        }
        return "ENT-";
    }    

    private String getEntitlementLineItemId(EntitlementLineItem lineItem)  {
        
        List<Product> products = lineItem.getEntitledProducts();
        LicenseModel model = lineItem.getLicenseModel();
        
        if (products != null && products.size() >= 1 && model != null)  {
            String prdName = products.get(0).getName();
            String modelName = model.getName();
            StringBuilder builder = new StringBuilder();
            builder.append(getMagicChars(prdName));
            builder.append(getMagicChars(modelName) + '-');
            return builder.toString();
            
        }
        return "ACT-";
    }
    
    private String getBulkEntitlementId(BulkEntitlement blk)  {
        List<Product> products = blk.getEntitledProducts();
        List<ChannelPartner> partners= blk.getEntChannelPartners();
        OrganizationUnit ou = blk.getSoldTo();
        
        if (ou != null )  {
            return getMagicChars(ou.getName()) + '-';
        }
        else if (partners != null && partners.size() >= 1)  {
            return getMagicChars(partners.get(0).getContact().getFirstName()) + '-';
        }
        else if (products != null && products.size() >= 1)  {
            return getMagicChars( products.get(0).getName()) + '-';
        }
        return "BLK-";
    }
    
    private String getMaintenanceItemID(MaintenanceItem main)  {
        Product mProd = main.getMaintenanceProduct();
        PartNumber mPart = main.getPartNumber();
        if (mPart != null)
            return getMagicChars( mPart.getPartNumberName()) + '-';
        else if (mProd != null)
            return getMagicChars( mProd.getName()) + '-';
        return "MIANT-";
    }
    
    private String getFulfillmentId(FulfillmentRecord fr)  {
        EntitlementLineItem lineItem = fr.getLineItem();
        if (lineItem != null)  {
            Entitlement ent = lineItem.getParentEntitlement();
            if (ent != null)  {
                OrganizationUnit ou = ent.getSoldTo();
                List<Product> products  = lineItem.getEntitledProducts();
                if (ou != null && products != null && products.size() >= 1 )  {
                    return getMagicChars(ou.getName()) + '-' + getMagicChars(products.get(0).getName()) + '-';
                }
            }
        }
        
        return "FID-";
    }
    
    private String getConsolidatedLicenseID(ConsolidatedLicenseRecord conId)  {
        OrganizationUnit ou = conId.getSoldTo();
        if (ou != null) 
            return getMagicChars("CON" + ou.getName()) + '-';
        return "CID-";
            
    }
    
    private SvcException getFaultInfo(String msg)  {
        SvcException faultInfo = new SvcException();
        if (msg != null)
            faultInfo.setMessage(msg);
        else 
            faultInfo.setMessage("(null)");
        return faultInfo;
    }
    
    @Override
    public PingResponse ping(PingRequest request) throws IdGeneratorException {
        PingResponse resp = new PingResponse();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
        resp.setStr(" Id Generator version : 1.0.1\n Token : " + request.getStr());
        resp.setProcessedTime("" + dateFormat.format(new Date()));
        return resp;
    }

    @Override
    public Id generateEntitlementID(Entitlement entitlement) throws IdGeneratorException {
        try  {
            String prefix = entitlement.isTest() ? "TES-" : "";
            Id id = new Id();
            id.setId(generateUniqueRandom(prefix + getEntitlementId(entitlement), randomGenerator.nextInt(65535)));
            return id;
        }catch (Exception ex)  {
            ex.printStackTrace();
            throw new IdGeneratorException("ID_GEN_EXCEPTION", getFaultInfo(ex.getMessage()));
        }

    }
    @Override
    public Id generateLineItemID(EntitlementLineItem entitlementLineItem)
            throws IdGeneratorException {
        try  {
            String prefix = entitlementLineItem.isTest() ? "TES-" : "";
            Id id = new Id();
            id.setId(generateUniqueRandom(prefix + getEntitlementLineItemId(entitlementLineItem), randomGenerator.nextInt(65535)));
            return id;
        }catch (Exception ex)  {
            ex.printStackTrace();      
            throw new IdGeneratorException("ID_GEN_EXCEPTION", getFaultInfo(ex.getMessage()));
        }    
   }

    @Override
    public Id generateWebRegKey(BulkEntitlement bulkEntitlement) throws IdGeneratorException {
        try  {
            String prefix = bulkEntitlement.isTest() ? "TES-" : "";
            Id id = new Id();
            id.setId(generateUniqueRandom(prefix + getBulkEntitlementId(bulkEntitlement), randomGenerator.nextInt(65535)));
            return id;
        }catch (Exception ex)  {
            ex.printStackTrace();      
            throw new IdGeneratorException("ID_GEN_EXCEPTION", getFaultInfo(ex.getMessage()));
        }
    }



    @Override
    public Id generateMaintenanceItemID(MaintenanceItem maintenanceItem)
            throws IdGeneratorException {
        try  {
            Id id = new Id();
            id.setId(generateUniqueRandom("" + getMaintenanceItemID(maintenanceItem), randomGenerator.nextInt(65535)));
            return id;
        }catch (Exception ex)  {
            ex.printStackTrace();      
            throw new IdGeneratorException("ID_GEN_EXCEPTION", getFaultInfo(ex.getMessage()));
        }
    }


    @Override
    public Id generateFulfillmentID(FulfillmentRecord fulfillmentRecord)
            throws IdGeneratorException {
        try  {
            Id id = new Id();
            id.setId(generateUniqueRandom("" + getFulfillmentId(fulfillmentRecord), randomGenerator.nextInt(65535)));
            return id;
        }catch (Exception ex)  {
            ex.printStackTrace();      
            throw new IdGeneratorException("ID_GEN_EXCEPTION", getFaultInfo(ex.getMessage()));
        }
    }


    @Override
    public Id generateConsolidatedLicenseID(ConsolidatedLicenseRecord consolidatedLicenseRecord)
            throws IdGeneratorException {
        try  {
            Id id = new Id();
            id.setId(generateUniqueRandom("" + getConsolidatedLicenseID(consolidatedLicenseRecord), randomGenerator.nextInt(65535)));
            return id;
        }catch (Exception ex)  {
            ex.printStackTrace();      
            throw new IdGeneratorException("ID_GEN_EXCEPTION", getFaultInfo(ex.getMessage()));
        }
    }

}
