package com.flexnet.external.webservice.keygenerator;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.jws.WebService;

import com.flexnet.external.type.ConsolidatedLicense;
import com.flexnet.external.type.ConsolidatedLicenseResquest;
import com.flexnet.external.type.CustomAttribute;
import com.flexnet.external.type.CustomAttributeDescriptor;
import com.flexnet.external.type.CustomHostId;
import com.flexnet.external.type.Feature;
import com.flexnet.external.type.FeatureOverride;
import com.flexnet.external.type.FulfillmentRecord;
import com.flexnet.external.type.FulfillmentRecordSet;
import com.flexnet.external.type.GeneratorRequest;
import com.flexnet.external.type.GeneratorResponse;
import com.flexnet.external.type.HostIdRequest;
import com.flexnet.external.type.LicenseFileDefinition;
import com.flexnet.external.type.LicenseFileDefinitionMap;
import com.flexnet.external.type.LicenseFileDefinitionMapItem;
import com.flexnet.external.type.LicenseFileMapItem;
import com.flexnet.external.type.LicenseGeneratorConfig;
import com.flexnet.external.type.LicenseModel;
import com.flexnet.external.type.LicenseModelRequest;
import com.flexnet.external.type.LicenseTechnology;
import com.flexnet.external.type.OrgUnitUser;
import com.flexnet.external.type.OrganizationUnit;
import com.flexnet.external.type.PartNumber;
import com.flexnet.external.type.PingRequest;
import com.flexnet.external.type.PingResponse;
import com.flexnet.external.type.Product;
import com.flexnet.external.type.ProductRequest;
import com.flexnet.external.type.Status;
import com.flexnet.external.type.StringPair;
import com.flexnet.external.type.SuiteProductInfo;
import com.flexnet.external.type.SvcException;



@WebService(
    endpointInterface="com.flexnet.external.webservice.keygenerator.LicenseGeneratorServiceInterface",
    wsdlLocation = "/schema/LicenseGeneratorService.wsdl"
)

public class  LicenseGeneratorServiceImpl implements LicenseGeneratorServiceInterface{

    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
    private String getLicenseStorageType(String name, LicenseTechnology technology)  {
        if (technology != null)  {
            List <LicenseFileDefinition> licDefs = technology.getLicenseFileDefinitions();
            if (licDefs != null)  {
                for (LicenseFileDefinition item : licDefs)    {
                    if (item.getName().equals("name"))  {
                        return item.getLicenseStorageType().toString();
                    }
                }
            }
        }
        return "TEXT";
    }

    private void printCustomLicenseAttribute(List<CustomAttributeDescriptor> attribute, StringBuffer result, String type)  {
        result.append("\n\t " + type + " Custom License Attribute ");
        if (attribute != null)  {
            for (CustomAttributeDescriptor attr : attribute)  {
                result.append("\n\t\t" + attr.getName() + ": " + attr.getValue());
            }
        }
    }

    private void printCustomAttribute(List<CustomAttribute> attribute, StringBuffer result, String type)  {
        result.append("\n\t " + type + " Attribute");
        if (attribute != null)  {
            for (CustomAttribute attr : attribute)  {
                result.append("\n\t\t " + attr.getName()).append(": ");
                if (attr.getAttributes() != null)  {
                for (String value : attr.getAttributes())  {
                     result.append(value + ",");
                }
                }
            }
        }
    }

    private void printFeature(List<Feature> features, StringBuffer result)  {
        result.append("\n\t Features");
        if (features != null)  {
            for (Feature feature : features)  {
                result.append("\n\t\t " + feature.getName() + " : " + feature.getCount() + ", version : " + feature.getVersion() + ", isDateBased : "+ feature.isIsDateBasedVersion()+ "\n\t");
                List<FeatureOverride> overrides = feature.getFeatureOverride();
                result.append("\n\t\t Featur Overrides" );
                for (FeatureOverride fos : overrides)  {
                    result.append("\n\t\t\t " + fos.getName() + " : " + fos.getValue());
                }
            }
        }
    }

    private void printCustomHost(CustomHostId custHost, StringBuffer result)  {
        result.append("\n\t CustomHost");
        if(custHost != null)            {
            result.append("\n\t\t HostId : " + custHost.getHostIdentifier());
            result.append("\n\t\t License Technology : " + custHost.getLicenseTechnologyName());
            result.append("\n\t\t Custom License Host Attributes");
            List<StringPair> attrVals = custHost.getHostAttributeValues();
            for (StringPair pair : attrVals)  {
                result.append("\n\t\t\t " +  pair.getKey() +" : " + pair.getValue());
            }

        }
    }

    private void printFulfillments(List<FulfillmentRecord> pFrs, StringBuffer result)  {
        if (pFrs != null && pFrs.size() > 0)  {
           result.append("\nParent Fulfillments\n");
           for (FulfillmentRecord fr : pFrs)  {
               result.append("\nFulfillment Id : ").append(fr.getFulfillmentId());
               result.append("\n Returned Count : ").append(fr.getReturnedCount());
           }
        }
    }

    private void printOrg(OrganizationUnit org, StringBuffer result)  {
        result.append("\n\t OrganizationUnit");
        if (org != null)  {
            result.append("\n\t\t DisplayName : " + org.getDisplayName());
            printCustomAttribute(org.getCustomAttributes(), result, "OrganizationUnit");
        }
    }


    private void printUsers(List<OrgUnitUser> users, StringBuffer result, String type)  {
        if (users != null && users.size() > 0)  {
            result.append("\n\t " + type);
           for (OrgUnitUser user : users)  {
               result.append("\n\t\t DisplayName : " + user.getDisplayName());
               result.append("\n\t\t FirstName   : " + user.getFirstName());
               result.append("\n\t\t LastName    : " +  user.getLastName());
               printCustomAttribute(user.getCustomAttributes(), result, "User");
           }
        }
    }

    private void printProduct(List<Product> products, StringBuffer result)  {
        if (products != null && products.size() > 0)  {
           result.append("\nProduct" );
           for (Product product : products)  {
               result.append("\n\t Name    : " + product.getName());
               result.append("\n\t Version : " + product.getVersion());
               result.append("\n\t Quantity Per Copy : " + product.getQuantityPerCopy());
               if (product.isSuite())  {
                    printSuiteProductInfo(product.getSuiteProductInfo(), result);
               }
               else {
                    printCustomAttribute(product.getCustomAttributes(), result, "Product");
               }
               printFeature(product.getFeatures(), result);

           }
        }

    }

    private void printSuiteProductInfo(List<SuiteProductInfo> products, StringBuffer result)  {
        if (products != null)  {
            for (SuiteProductInfo product : products)  {
                result.append("\n\t Suite Product");
                result.append("\n\t\t Name    : " + product.getName());
                result.append("\n\t\t Version : " + product.getVersion());
                result.append("\n\t\t Quantity : " + product.getQuantity());
                if (product.getPackageName() != null && product.getPackageName().equals(""))  {
                    result.append("\n\t\t Package Name : " + product.getPackageName());
                    result.append("\n\t\t Package Version : " + product.getPackageVersion());
                    result.append("\n\t\t Package Version Format : " + product.getPackageVersionFormat());
                }
                printCustomAttribute(product.getCustomAttributes(), result, "Suite Product");

            }
        }
    }





    @Override
    public PingResponse ping(PingRequest request) {
        PingResponse resp = new PingResponse();
        resp.setStr(" Key generator version : 1.0.0\n Token : " + request.getStr());
        resp.setProcessedTime("" + dateFormat.format(new Date()));
        return resp;
    }


    @Override
    public Status validateProduct(ProductRequest product) throws LicGeneratorException {

        if (product.getName().isEmpty() || product.getVersion().isEmpty())  {
            SvcException fault  = new SvcException();
            fault.setMessage("Invalid product ");
            throw new LicGeneratorException("INVALID_PRODUCT", fault);
        }
        System.out.println("server validated product " + product.getName());
        Status status = new Status();
        status.setCode(0);
        return status;
    }

    @Override
    public Status validateLicenseModel(LicenseModelRequest model) throws LicGeneratorException {
        if (model.getName().isEmpty() || model.getModelType().isEmpty())   {
            SvcException fault  = new SvcException();
            fault.setMessage("invalid license model ");
            throw new LicGeneratorException("INVALID_LICENSE_MODEL", fault);
        }
        System.out.println("server validated license model " + model.getName());
        Status status = new Status();
        status.setCode(0);
        return status;
    }

    @Override
    public GeneratorResponse generateLicense(GeneratorRequest request) throws LicGeneratorException {
        GeneratorResponse response = new GeneratorResponse();
        try {

            OrganizationUnit ou = request.getSoldTo();
            List <OrgUnitUser> soldTousers = request.getSoldToUsers();
            OrgUnitUser loggedInUser = request.getLoggedInUser();
            List<Product> products = request.getEntitledProducts();
            LicenseModel model = request.getLicenseModel();
            LicenseGeneratorConfig config = request.getLicenseGeneratorConfiguration();
            LicenseTechnology licTech = request.getLicenseTechnology();
            CustomHostId custHost  = request.getCustomHost();
            List<FulfillmentRecord> pFrs = request.getParentFulfillments(); // Parent fulfillment
            PartNumber partNo = request.getPartNumber();

            List<LicenseFileDefinition> lfds = licTech.getLicenseFileDefinitions();
            SimpleDateFormat sdf = new SimpleDateFormat("MM-dd-yyyy");

            StringBuffer result = new StringBuffer("Fulfillment\n");

            // Start from Fulfilment
            result.append("\t fulfilment Id: " + request.getFulfilementID());
            result.append("\n\t Count: " + request.getFulfillCount());
            result.append("\n\t OverDraft : " + request.getOverdraftCount());
            if (request.getStartDate() != null && request.getStartDate().toGregorianCalendar() != null )
                result.append("\n\t StartDate : " + sdf.format(request.getStartDate().toGregorianCalendar().getTime()));
            result.append("\n\t ShipToEmail: " + request.getShipToEmail());
            printCustomHost(custHost, result);
            printFulfillments(pFrs, result);
            result.append("\n\t RequestType : " + request.getRequestType());
            result.append("\n\t isVerifyRequest : " + request.isVerifyRequest());
             if (model.getFulfillmentTimeAttributes() != null)
                 printCustomLicenseAttribute(model.getFulfillmentTimeAttributes().getAttributes(), result, "FULFILMENT");

            // <!--========== Line item details =============== -->
            result.append("\nActivationRecord\n\t Activation ID: " + request.getActivationID());
            if (request.getVersionDate() != null && request.getVersionDate().toGregorianCalendar() != null)
                result.append("\n\t Version Date : " + sdf.format(request.getVersionDate().toGregorianCalendar().getTime()));
            if (request.getExpirationDate() != null && request.getExpirationDate().toGregorianCalendar() != null)
                result.append("\n\t ExpirationDate : " + sdf.format(request.getExpirationDate().toGregorianCalendar().getTime()));
            printCustomAttribute(request.getEntitlementLineItemCustomAttributes(), result, "LineItem");
            if (model.getEntitlementTimeAttributes() != null)
                printCustomLicenseAttribute(model.getEntitlementTimeAttributes().getAttributes(), result, "ENTITLEMENT");

            // <!--========== Entitlement details =============== -->

            result.append("\nEntitlement\n\t EntitlementId : " + request.getEntitlementID());
            result.append("\n\t OrderId : " + request.getOrderId());
            result.append("\n\t OrderLineNumber: " + request.getOrderLineNumber());
            printOrg(ou, result);
            printUsers(soldTousers, result, "SoldToUser");
            printCustomAttribute(request.getEntitlementCustomAttributes(), result, "Entitlement");


            // <!--========== Product details =============== -->
            printProduct(products, result);
            if (partNo != null)
                result.append("\n PartNumber: " + partNo.getPartNumberName());

            // <!--========== License Model details =============== -->
             result.append("\nLicense\n\t Model Name: " + model.getName());
             if (model.getModelTimeAttributes() != null)
                 printCustomLicenseAttribute(model.getModelTimeAttributes().getAttributes(), result, "MODEL");

            // <!--========== License Generator Config details =============== -->
             if(config != null)  {
                 result.append("\nLicense Generator Config\n\t Name: " + config.getName());
                 if (config.getAttributes() != null )
                     printCustomLicenseAttribute(config.getAttributes().getAttributes(), result, "Generator");
             }

             // <!--========== License Technology details =============== -->
             result.append("\nLicense Technolgy \n\t Name: " + licTech.getName());
             if (lfds != null && lfds.size() > 0)  {
                 for (LicenseFileDefinition lfd :lfds)  {
                     result.append("\n\t License file defnition Name : " + lfd.getName());
                     result.append("\n\t License file defnition Type : " + lfd.getLicenseStorageType().toString());
                 }
             }
             result.append("\n\t LicenseFileType : " + request.getLicenseFileType());


             // <!--========== Misc. details =============== -->
             //request.getCurrentLicenseOnHost();
             result.append("\nLogged In User : ").append(loggedInUser.getDisplayName());
             printCustomAttribute(loggedInUser.getCustomAttributes(), result, "LoggedIn User");


            // TODO: populate the license files in the response as per the license file definitions
            //LicenseFileMap licFiles = new LicenseFileMap();

            List<LicenseFileMapItem> licFiles = new ArrayList<LicenseFileMapItem>();
            int i = 0;
            for (LicenseFileDefinition lfd : lfds)  {
                if (lfd.getLicenseStorageType().toString().equals("TEXT"))  {
                    LicenseFileMapItem lf = new LicenseFileMapItem();
                    lf.setName(lfd.getName());
                    lf.setValue(result.toString());
                    licFiles.add((lf));
                }
                else if (lfd.getLicenseStorageType().toString().equals("BINARY"))  {
                    StringBuilder tmp = new StringBuilder();
                    MessageDigest md = MessageDigest.getInstance("SHA-256");
                    md.update(result.toString().getBytes("UTF-8")); // Change this to "UTF-16" if needed
                    byte[] digest = md.digest();
                    StringBuffer hexString = new StringBuffer();
                    for (int j=0;j<digest.length;j++) {
                      hexString.append(Integer.toHexString(0xFF & digest[j]));
                    }
                    tmp.append("License       : " + hexString.toString() + "\n");
                    LicenseFileMapItem lf = new LicenseFileMapItem();
                    lf.setName(lfd.getName());
                    lf.setValue(tmp.toString().getBytes("UTF-8"));
                    licFiles.add((lf));
                }
                i++;
            }
            response.setLicenseFiles(licFiles);
            StringBuilder tmp = new StringBuilder(); // Using default 16 character size


            tmp.append("License Model : " + request.getLicenseModel().getModelType() + "\n" );
            String licenseText = tmp.toString();


            MessageDigest md = MessageDigest.getInstance("SHA-256");
            md.update(licenseText.getBytes("UTF-8")); // Change this to "UTF-16" if needed
            byte[] digest = md.digest();
            StringBuffer hexString = new StringBuffer();

            for (i=0;i<digest.length;i++) {
              hexString.append(Integer.toHexString(0xFF & digest[i]));
            }

            tmp.append("License       : " + hexString.toString() + "\n");
            tmp.append("PreviousSample \n" + result.toString() + "\n" );
            response.setLicenseText("#Generated by external server at "  + dateFormat.format(new Date()) + "\n" + tmp.toString());
            response.setLicenseFileName("nofile.txt");

            System.out.println("Processed generateLicense at " + dateFormat.format(new Date()));
        }catch(Exception e){
            String msg = e.getMessage();
            if (msg == null)
                msg = "unknown error; possibly NPE";
            System.out.println("Error " + msg);
            e.printStackTrace();
            SvcException fault  = new SvcException();
            fault.setMessage("msg");
            throw new LicGeneratorException("ERROR_TO_BE_DEFINED", fault);
        }
        return response;
    }

    @Override
    public ConsolidatedLicense consolidateFulfillments(FulfillmentRecordSet fulfillmentRecordset)
            throws LicGeneratorException {

        if (fulfillmentRecordset == null)
            return null;
        List<FulfillmentRecord> fulfillments = fulfillmentRecordset.getFulfillments();
        if (fulfillments == null || fulfillments.size() == 0)
            return null;

        FulfillmentRecord fr = fulfillments.get(0);
        ConsolidatedLicense response = new ConsolidatedLicense();

        StringBuffer sb = new StringBuffer();

        fr.getLicenseTechnology().getLicenseFileDefinitions();
        for (int ii = 0; ii < fulfillments.size(); ii++) {
            FulfillmentRecord tfr = fulfillments.get(ii);
            List<LicenseFileMapItem> files = tfr.getLicenseFiles();
            if (files != null)  {
                for (LicenseFileMapItem item : files)    {
                    String storageType = getLicenseStorageType(item.getName(), tfr.getLicenseTechnology());
                    if (storageType.toString().equals("TEXT"))  {
                        sb.append(item.getValue() + "\n");
                    }
                    if (storageType.toString().equals("BINARY"))  {
                        sb.append(item.getValue());
                    }
                }
            }
        }

        List<LicenseFileDefinitionMapItem> licenseFilenames = new ArrayList<LicenseFileDefinitionMapItem>();
        List<LicenseFileMapItem> licenseFiles = new ArrayList<LicenseFileMapItem>();
        //LicenseFileMap licenseFiles = new LicenseFileMap();
        LicenseTechnology licTech = fr.getLicenseTechnology();
        List<LicenseFileDefinition> lfds = licTech.getLicenseFileDefinitions();

        //List<LicenseFileDefinitionMapItem> licFileNames = new LicenseFileDefinitionMapItem[lfds.length];
        //LicenseFileMapItem[] licFiles = new LicenseFileMapItem[lfds.length];

        for (LicenseFileDefinition lfd : lfds)  {
            try {
                LicenseFileDefinitionMapItem lfdItem = new LicenseFileDefinitionMapItem();
                LicenseFileMapItem lfItem = new LicenseFileMapItem();
                if (lfd.getLicenseStorageType().toString().equals("BINARY"))   {
                    lfdItem.setName(lfd.getName());
                    lfdItem.setValue("BIN_CON_" + fr.getFulfillmentId() + ".lic");

                    lfItem.setName(lfd.getName());
                    lfItem.setValue(sb.toString().getBytes("UTF-8"));
                }
                else   {
                    lfdItem.setName(lfd.getName());
                    lfdItem.setValue("CON_" + fr.getFulfillmentId() + ".lic");

                    lfItem.setName(lfd.getName());
                    lfItem.setValue(sb.toString());

                }
                licenseFiles.add(lfItem);
                licenseFilenames.add(lfdItem);
            } catch (UnsupportedEncodingException e) {    }
        }

        try {
            response.setBinaryLicense(sb.toString().getBytes("UTF-8"));
        } catch (UnsupportedEncodingException e) {    }
        response.setCriteria("CustomConsolidationCriteria");
        response.setFulfillments(fulfillments);
        response.setLicenseFileName("consolidatedTextLicense" + fulfillments.get(0).getFulfillmentId() + ".lic");
        response.setLicenseFileNames(licenseFilenames);
        response.setLicenseText(sb.toString());
        response.setLicFiles(licenseFiles);

        return response;
    }

    @Override
    public LicenseFileDefinitionMap generateLicenseFilenames(GeneratorRequest fileRec)
            throws LicGeneratorException {
        LicenseTechnology licTech = fileRec.getLicenseTechnology();
        List<LicenseFileDefinition> lfds = licTech.getLicenseFileDefinitions();

        StringBuilder result = new StringBuilder();
        List<LicenseFileDefinitionMapItem> mItem = new ArrayList<LicenseFileDefinitionMapItem>();


        result.append(fileRec.getOrderId() + "-" + System.currentTimeMillis());
        for (LicenseFileDefinition lfd : lfds)  {
            LicenseFileDefinitionMapItem item = new LicenseFileDefinitionMapItem();
            item.setName(lfd.getName());
            if (lfd.getLicenseStorageType().toString().equals("TEXT"))  {
                item.setValue(lfd.getName() + "-" + result.toString() + ".txt");
            }
            else if (lfd.getLicenseStorageType().toString().equals("BINARY"))   {
                item.setValue(lfd.getName() + "-Bin-"+ result.toString());
            }
            mItem.add(item);

        }
        System.out.println("Processed generateFulfillmentLicenseFilenames at " + dateFormat.format(new Date()));

        LicenseFileDefinitionMap t = new LicenseFileDefinitionMap();
        t.setItem(mItem);
        return t;
    }

    @Override
    public LicenseFileDefinitionMap generateConsolidatedLicenseFilenames(
            ConsolidatedLicenseResquest clRec) throws LicGeneratorException {
       List<FulfillmentRecord> fulfillments = clRec.getFulfillments();

        if (fulfillments != null && fulfillments.size() > 0)  {

            LicenseTechnology licTech = fulfillments.get(0).getLicenseTechnology();
            List<LicenseFileDefinition> lfds = licTech.getLicenseFileDefinitions();
            List<LicenseFileDefinitionMapItem> items = new ArrayList<LicenseFileDefinitionMapItem>();

            for (LicenseFileDefinition lfd : lfds)  {
                LicenseFileDefinitionMapItem item = new LicenseFileDefinitionMapItem();
                item.setName(lfd.getName());

                if (lfd.getLicenseStorageType().toString().equals("TEXT")) {
                    item.setValue(lfd.getName() + "-CON-" + fulfillments.get(0).getFulfillmentId()+".txt");
                } else {
                    item.setValue(lfd.getName() + "-CON-Bin-" + fulfillments.get(0).getFulfillmentId());
				}
				items.add(item);
            }
            System.out.println("Processed generateConsolidatedLicenseFilenames at " + dateFormat.format(new Date()));
            LicenseFileDefinitionMap t = new LicenseFileDefinitionMap();
            t.setItem(items);
            return t;

        }
        return null;
    }

    @Override
    public String generateCustomHostIdentifier(HostIdRequest hostIdReq)
            throws LicGeneratorException {
        if (hostIdReq != null)  {
            List<StringPair> hostIdMap = hostIdReq.getHostIdMap();
            String hostIdType = hostIdReq.getHostIdType();
            if (hostIdMap != null)   {
                String fid = hostIdType;
                for (StringPair item : hostIdMap)  {
                    fid = item.getKey() +"="+ item.getValue();
                }
                System.out.println("Processed generateCustomHostIdentifier at " + dateFormat.format(new Date()));
                 return hostIdType + ":"+fid;
            }
        }
        return null;
    }
}
