

package com.flexnet.external.webservice.renewal;

import java.util.List;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.jws.WebService;

import com.flexnet.external.type.CustomAttribute;
import com.flexnet.external.type.Feature;
import com.flexnet.external.type.FeatureOverride;
import com.flexnet.external.type.PingRequest;
import com.flexnet.external.type.PingResponse;
import com.flexnet.external.type.Product;
import com.flexnet.external.type.RenewableEntitlementLineItem;
import com.flexnet.external.type.RenewableEntitlementLineItems;
import com.flexnet.external.type.RenewalResponse;
import com.flexnet.external.type.RenewalStatus;
import com.flexnet.external.type.SuiteProductInfo;



@WebService(
    endpointInterface="com.flexnet.external.webservice.renewal.RenewalServiceInterface",
    wsdlLocation = "/schema/RenewalService.wsdl"
)


public class RewnewalServiceImpl implements RenewalServiceInterface{

	@Override
	public PingResponse ping(PingRequest request) throws RenewalSeviceException {
        PingResponse resp = new PingResponse();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
        resp.setStr("Renewal Service version : 1.0.0");
        resp.setProcessedTime("" + dateFormat.format(new Date()));
        return resp;
	}

	@Override
	public RenewalResponse request(RenewableEntitlementLineItems renewableEntitlementLineItems)
			throws RenewalSeviceException {
		RenewalResponse response = new RenewalResponse();
		response.setStatus(RenewalStatus.FAILURE);
		List<RenewableEntitlementLineItem> requestedLineItems = renewableEntitlementLineItems.getRenewableEntitlementLineItems();
		if (requestedLineItems != null && requestedLineItems.size() > 0)  {
			StringBuffer str = new StringBuffer();
			for (RenewableEntitlementLineItem req : requestedLineItems)  {
				str.append("LineItem " + req.getActivationID());
                str.append("\nType " + req.getType());
                str.append("\nRemainingDays " + req.getRemainingDays());
                str.append("\nSoldTo " + req.getOrgName());
                str.append("\nNo of Sold Copies " + req.getNumberOfCopies());
                str.append("\nNo of Used copies " + req.getFulfilledAmount());
                str.append("\nNo of Unused copies " + req.getNumberOfRemainingCopies());
                str.append("\nOrder Id " + req.getOrderId());
                str.append("\nExpiring Date " + req.getLicenseTerm().getExpirationDate().toGregorianCalendar().getTime());

                str.append("\nEntitlement " + req.getParentEntitlement().getEntitlementID());
                
                getCustomAttribute(req.getParentEntitlement().getCustomAttributes(), str, "Oreder");
                getCustomAttribute(req.getCustomAttributes(), str, "LineItem");
				
                getProduct(req.getEntitledProducts(), str);
                
                System.out.println("Renewal request for " + str.toString());
                
			}
			response.setStatus(RenewalStatus.SUCCESS);
		}
		return response;
	}
   
    private void getCustomAttribute(List<CustomAttribute> attribute, StringBuffer result, String type)  {
        result.append("\n\t " + type + " Attribute");
        if (attribute != null)  {
            for (CustomAttribute attr : attribute)  {
                result.append("\n\t\t " + attr.getName()).append(": ");
                if (attr.getAttributes() != null)  {
                for (String value : attr.getAttributes())  {
                     result.append(value + ",");
                }
                }
            }
        }
    }
    
    private void getProduct(List<Product> products, StringBuffer result)  {
        if (products != null && products.size() > 0)  {
           result.append("\nProduct" );
           for (Product product : products)  {
               result.append("\n\t Name    : " + product.getName());
               result.append("\n\t Version : " + product.getVersion());
               result.append("\n\t Quantity Per Copy : " + product.getQuantityPerCopy());
               if (product.isSuite())  {
                    getSuiteProductInfo(product.getSuiteProductInfo(), result);
               }
               else {
                    getCustomAttribute(product.getCustomAttributes(), result, "Product");
               }
               getFeature(product.getFeatures(), result);

           }
        }

    }
    

    private void getSuiteProductInfo(List<SuiteProductInfo> products, StringBuffer result)  {
        if (products != null)  {
            for (SuiteProductInfo product : products)  {
                result.append("\n\t Suite Product");
                result.append("\n\t\t Name    : " + product.getName());
                result.append("\n\t\t Version : " + product.getVersion());
                result.append("\n\t\t Quantity : " + product.getQuantity());
                if (product.getPackageName() != null && product.getPackageName().equals(""))  {
                    result.append("\n\t\t Package Name : " + product.getPackageName());
                    result.append("\n\t\t Package Version : " + product.getPackageVersion());
                    result.append("\n\t\t Package Version Format : " + product.getPackageVersionFormat());
                }
                getCustomAttribute(product.getCustomAttributes(), result, "Suite Product");

            }
        }
    }
    
    private void getFeature(List<Feature> features, StringBuffer result)  {
        result.append("\n\t Features");
        if (features != null)  {
            for (Feature feature : features)  {
                result.append("\n\t\t " + feature.getName() + " : " + feature.getCount() + ", version : " + feature.getVersion() + ", isDateBased : "+ feature.isIsDateBasedVersion());
                List<FeatureOverride> overrides = feature.getFeatureOverride();
                if (overrides != null && overrides.size()>0)   {
	                result.append("\n\t\t Feature Overrides" );
	                for (FeatureOverride fos : overrides)  {
	                    result.append("\n\t\t\t " + fos.getName() + " : " + fos.getValue());
	                }
                }
            }
        }
    }

}
